;##################################################################
; CoolDlg
;##################################################################

#include CoolDlg.h

#DEFINE TITLEBARHEIGHT	40
#DEFINE CLOSEBTNHEIGHT	18
#DEFINE ID_CLOSEBTN 9999

DATA SECTION
	// Instance handle for this process
	hInstance		HANDLE	?
	// Handle of the main dialog
	hDlg			HANDLE	?
	// Flag to indicate the the window is being dragged
	IsDragging		BOOL	?

	// Holder for POINT that indicates how far the cursor is
	// from the top left corner of the window
	OffsetPoint		POINT	<?>

	hCloseBmp		HANDLE	?

	hRegion			HANDLE	?

CODE SECTION

START:
	invoke GetModuleHandle, 0
	mov [hInstance],eax
	invoke DialogBoxParam,[hInstance],1000,0,ADDR DlgProc,0
	invoke ExitProcess,0

DlgProc FRAME hwnd,uMsg,wParam,lParam
	uses ebx
	LOCAL ptc:POINT
	LOCAL rct:RECT
	LOCAL ps:PAINTSTRUCT
	LOCAL hrgn:%HANDLE
	LOCAL hmemdc:%HANDLE
	LOCAL hbmp:%HANDLE
	LOCAL btnrct:RECT

	.WM_MOUSEMOVE
	cmp D[uMsg],WM_MOUSEMOVE
	jne >>.WM_PAINT
		// If the window is dragging then move it with the mouse
		cmp D[IsDragging],TRUE
		jne >>.DEFPROC
		// Get the current cusor position and adjust it to
		// the top left corner of the window
		invoke GetCursorPos,offset ptc
		mov eax,[OffsetPoint.x]
		sub [ptc.x],eax
		mov eax,[OffsetPoint.y]
		sub [ptc.y],eax
		// Move the window to its new position
		invoke SetWindowPos,[hwnd],0,[ptc.x],[ptc.y],0,0,SWP_NOSIZE + SWP_NOZORDER

		xor eax,eax
		ret

	.WM_PAINT
	cmp D[uMsg],WM_PAINT
	jne >>.WM_LBUTTONDOWN
		// The window is painted here, call BeginPaint to start
		invoke BeginPaint,[hwnd],offset ps

			// Make it pretty

			// Get the size of the window. Since there's no non-client area
			// the total size is equal to the client area
			invoke GetClientRect,[hwnd],offset rct

			// A memory DC is used for our drawing
			invoke CreateCompatibleDC,[ps.hdc]
			mov [hmemdc],eax
			// Create a bitmap to draw on
			invoke CreateCompatibleBitmap,[ps.hdc],[rct.right],[rct.bottom]
			mov [hbmp],eax
			// Select the bitmap into the memory DC and push the old bitmap object handle
			invoke SelectObject,[hmemdc],eax
			push eax

			// Call the fill procedure to paint the bitmap
			invoke GradientFillRect,[hmemdc],0FFFFF8F0h,0FFC0C0C0h,0,0,[rct.right],[rct.bottom],1
			push [rct.right]
			push [rct.bottom]
			// Draw a thin edge to indicate the new title bar
			// The window will only move if dragged from within
			// the new title area - See WM_LBUTTONDOWN for details
			mov D[rct.left],0
			mov D[rct.top],TITLEBARHEIGHT
			mov D[rct.bottom],TITLEBARHEIGHT
			invoke DrawEdge,[hmemdc],OFFSET rct,EDGE_ETCHED,BF_TOP

			invoke LoadIcon, NULL, IDI_APPLICATION
			invoke DrawIconEx,[hmemdc],10,5,eax,(TITLEBARHEIGHT-10),(TITLEBARHEIGHT-10),NULL,NULL,DI_NORMAL

			pop [rct.bottom]
			pop [rct.right]

			// Blt the backbuffer to the window
			invoke BitBlt,[ps.hdc],0,0,[rct.right],[rct.bottom],[hmemdc],0,0,SRCCOPY

			// pop the old bitmap object handle
			pop eax
			// Select it into the DC, this will release hbmp and return its handle
			invoke SelectObject,[hmemdc],eax

			// Clean up the objects we used in the paint process
			invoke DeleteObject,[hbmp]
			invoke DeleteDC,[hmemdc]
		// Shut down the painting and return FALSE
		invoke EndPaint,[hwnd],offset ps
		xor eax,eax
		ret

	.WM_LBUTTONDOWN
	cmp D[uMsg],WM_LBUTTONDOWN
	jne >>.WM_LBUTTONUP

		// The left button is down check to see if its
		// in the new title area
		invoke GetCursorPos,offset ptc
		invoke ScreenToClient,[hwnd],offset ptc
		cmp D[ptc.y],TITLEBARHEIGHT
		jg >>.DEFPROC

		invoke SetCapture,[hwnd]
		// The left button has been pressed in the new title 
		// area of the main window.
		// Calculate the offset from the top-left corner and
		// save the result in OffsetPoint. Use screen coordinates
		// for the calculation.

		invoke GetCursorPos,offset OffsetPoint

		invoke GetWindowRect,[hwnd],offset rct
		mov eax,[OffsetPoint.x]
		sub eax,[rct.left]
		mov [OffsetPoint.x],eax
		mov eax,[OffsetPoint.y]
		sub eax,[rct.top]
		mov [OffsetPoint.y],eax

		// Set the dragging flag to TRUE, any mouse movement
		// will cause the window to move as well until this
		// flag is reset when the mouse button is released
		mov D[IsDragging],TRUE
		xor eax,eax
		ret

	.WM_LBUTTONUP
	cmp D[uMsg],WM_LBUTTONUP
	jne >>.WM_INITDIALOG
		// Reset the dragging flag
		cmp D[IsDragging],FALSE
		je >>.DEFPROC
		invoke SetCapture,NULL
		mov D[IsDragging],FALSE
		xor eax,eax
		ret

	.WM_INITDIALOG
	cmp D[uMsg],WM_INITDIALOG
	jne >>.WM_COMMAND
		invoke GetClientRect,[hwnd],offset rct
		invoke CreateRoundRectRgn,0,0,[rct.right],[rct.bottom],10,10
		invoke SetWindowRgn,[hwnd],eax,TRUE

		invoke CreateWindowEx,NULL,"BUTTON",NULL,BS_BITMAP + BS_FLAT + BS_CENTER + \
			BS_PUSHBUTTON + WS_CHILD + WS_VISIBLE,0,0,40,CLOSEBTNHEIGHT,[hwnd],ID_CLOSEBTN,[hInstance],0
		mov ebx,eax

		invoke GetClientRect,ebx,offset btnrct
		mov eax,[rct.right]
		sub eax,[btnrct.right]
		sub eax,12
		invoke SetWindowPos,ebx,NULL,eax,0,0,0,SWP_NOSIZE + SWP_NOZORDER

		// Load the image for the close button
		invoke LoadImage,[hInstance],100,IMAGE_BITMAP,0,0,LR_DEFAULTCOLOR
		mov [hCloseBmp],eax

		// Assign the image to the button
		invoke SendMessage,ebx,BM_SETIMAGE,IMAGE_BITMAP,[hCloseBmp]

		jmp >.EXIT

	.WM_COMMAND
	cmp D[uMsg],WM_COMMAND
	jne >>.WM_CLOSE
		mov eax,[wParam]
		cmp eax,ID_CLOSEBTN
		jne >>.DEFPROC
			invoke DeleteObject,[hCloseBmp]
			invoke DeleteObject,[hRegion]
			invoke PostMessage,[hwnd],WM_CLOSE,0,0
			xor eax,eax
			ret

	.WM_CLOSE
	cmp D[uMsg],WM_CLOSE
	jne >.DEFPROC
		INVOKE EndDialog,[hwnd],0

	.DEFPROC
		mov eax,FALSE
		ret

	.EXIT

	mov eax, TRUE
	ret
ENDF

GradientFillRect FRAME hDC,Color1,Color2,x,y,w,h,Direction
	LOCAL vert[2]:TRIVERTEX
	LOCAL gRect:GRADIENT_RECT
	LOCAL gFillMethod:D

	mov eax,[x]
	mov [vert.x],eax
	mov eax,[y]
	mov [vert.y],eax

	mov eax,[Color1]

	xor ecx,ecx
	movzx ecx,al
	shl ecx,8
	mov [vert.Red],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Green],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Blue],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Alpha],cx

	mov eax,[w]
	add eax,[x]
	mov [vert.x + SIZEOF TRIVERTEX],eax
	mov eax,[h]
	add eax,[y]
	mov [vert.y + SIZEOF TRIVERTEX],eax

	mov eax,[Color2]

	xor ecx,ecx
	movzx ecx,al
	shl ecx,8
	mov [vert.Red + SIZEOF TRIVERTEX],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Green + SIZEOF TRIVERTEX],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Blue + SIZEOF TRIVERTEX],cx
	shr eax,8
	movzx ecx,al
	shl ecx,8
	mov [vert.Alpha + SIZEOF TRIVERTEX],cx

	mov D[gRect.UpperLeft],0
	mov D[gRect.LowerRight],1

	cmp D[Direction],FALSE
	jne >
		mov D[gFillMethod],GRADIENT_FILL_RECT_H
		jmp >.FILL
	:
		mov D[gFillMethod],GRADIENT_FILL_RECT_V
	.FILL

	invoke GradientFill,[hDC],ADDR vert,2,ADDR gRect,1,[gFillMethod]

	ret

ENDF
